	.length 59
	.GLOBAL TMSCODE	

; Programmer: Jed Margolin
;
; Date: 2/9/90
; PC Version
; Hardware: DSK
;-----------------------------------------------------------------------------
; Table memory:		
;
; Ram 256 words, 128 words direct, 2 pages
;-----------------------------------------------------------------------------
	.TITLE	"Ports and Zero Page Memory"
	.PAGE
; 320 Ports
PIN:		.SET	PA0 	; R, Interface Input

POUT:		.SET	PA0 	; W, Interface Output

STAT:		.SET	PA1	; R, Status Flags
				; 	d15 = 320FLAG
				; 	d14 = 68FLAG
				; 	d13 = CMD
				; 	d12 = M

XFLAG		.SET	PA1	; W, Extra Flag

temp		.SET	255
xtemp		.SET	254
xtemp2		.SET	253

; Zero Page RAM Variables
;		.=0000
	.bss 	zero,1
 	.bss	one,1
 	.bss	temp1,1
	.bss 	temp2,1
	.bss	padr,1
 	.bss	chksum,1
	.bss	speed,1
	.bss	cntr1,1
	.bss	cntr2,1
	.bss	allon,1
	.bss	bmask,1
	.bss	cmask,1
	.bss	mptr,1	
	.bss	input,1
	.bss	status,1
	.bss 	temp3,1
	.bss 	temp4,1
	.bss	buffer,16
	.bss	ctable,1
	.bss	wpair,1

	.GLOBAL	zero,one,temp1,temp2,padr,chksum,speed,cntr1,cntr2
	.GLOBAL	allon,bmask,cmask,mptr,input,status,temp3,temp4,buffer
;--------------------------------------------------------------------------
;	Startup
	.PAGE
	.sect "TMSCODE"
RESET:	B	START		; Reset Vector
INTR:	B	INT		; Interrupt Vector (Not used in this program)
;---------------------------------
; Data
CKSUM:	.WORD	(-0ACh+27h)&00FFh  ; makes the checksum turn out right; should
				; be 27h (Low byte only)
MSGADR:	.WORD	MSG		; Address pointer to Message
VERSION: .WORD	0013h		; version 1.3

; Table for Command Routines
CMDTBL:	.WORD	CMD0,CMD1,CMD2,CMD3,CMD4,CMD5,CMD6,CMD7
	.WORD	CMD8,CMD9,CMD10,CMD11,CMD12,CMD13,CMD14,CMD15,CMD16,CMD17
	.WORD	CMD18,CMD19,CMD20,CMD21,CMD22,CMD23,CMD24,CMD25,CMD26,CMD27

	.GLOBAL	CMD0,CMD1,CMD2,CMD3,CMD4,CMD5,CMD6,CMD7
	.GLOBAL	CMD8,CMD9,CMD10,CMD11,CMD12,CMD13,CMD14,CMD15,CMD16,CMD17
	.GLOBAL	CMD18,CMD19,CMD20,CMD21,CMD22,CMD23,CMD24,CMD25,CMD26,CMD27
	.GLOBAL COMMAND,SPIN
	.PAGE
; CMD0		; just echo the input port (Slow Echo)	
; CMD1		; just echo the input port (Fast Echo)	
; CMD2		; Checksum
; CMD3		; Version Number
; CMD4		; RAM Test will go here 
; CMD5		; SPIN (XFLAG Tone)
; CMD6		; Test 
; CMD7		; Idle and wait for Command 
; CMD8 - CMD27 	; Real Programs
;---------------------------------
INT:
	B	SPIN

;	SACL	cntr2		; special, to test Interrupts
;
;	LAC	cntr1
;	ADD	one
;	SACL	cntr1
;	OUT	cntr1,POUT
;
;	LAC	cntr2
;	EINT
;	RET
;---------------------------------
; From Reset
START:	LDPK	0		; Data Memory Page 0

	LACK	1
	SACL	one
;	SACL	cntr1		; special test only
; XMIT Test?
	IN	status,STAT
 	LAC	one,12		; d12 = M
	AND	status
	BZ	XMIT
;	EINT			; special test only
;=============================================================================
	.TITLE	"Commands"
	.PAGE
COMMAND:
	BIOZ	COMMAND		; Is the interface ready?

; Command ?
	IN	status,STAT
	LAC	one,13		; d13 = CMD
	AND	status
	BZ	SPIN		; no command, this is an error, so for now Spin
				; for development only
; Get the command and go to it
	IN	input,PIN
	LACK	0FFh			; mask
	AND	input
	SACL	temp1			; command is in low byte

	LACK	27
	SUB	temp1
	BLZ	SPIN			; development error, no command

	LACK	CMDTBL			; address of CMDTBL
	ADD	temp1			; table index (command)
	TBLR	mptr			; mptr points to the command routine

	LAC	mptr
	CALA				; Address comes from ACC

; If it ever comes back, just spin (This is an error, for development only)
	B	SPIN
;==============================================================================
; Toggles the XFLAG at 400Hz
SPIN:	ZAC
	SACL	zero
	SUB	one
	SACL	allon
SPINLP:	CALL	MWAIT
	OUT	zero,XFLAG
	CALL	MWAIT
	OUT	allon,XFLAG
	B	SPINLP
;=============================================================================
	.PAGE
; Command 0 is to just slow echo. Start by echoing 0000 
CMD0:	OUT	input,POUT
CM0A:	IN	status,STAT
	LAC	one,14
	AND	status
	BZ	CM0A

; we have a flag
	IN	input,PIN
	OUT	input,POUT
	B	CM0A	
;-----------------------------------------------------------------------------
; Command 1. Fast Echo
CMD1:	BIOZ	CMD1
	IN	temp1,PIN
	OUT	temp1,POUT
	B	CMD1
;-------------------------------------------------------------------------------
; Command 2. Do checksum
CMD2:	CALL	CHECKSUM
	B	COMMAND
;---------------------------------
; Command 3. Version Number
CMD3:	LACK	VERSION
	TBLR	temp1
	OUT	temp1,POUT
	B	COMMAND
;---------------------------------
; Command 4. RAM Test will go here
CMD4:	B	RAMTEST
;---------------------------------
; Command 5. Test the Error routine
CMD5:	B	ERRTST
;---------------------------------
; Command 6. Test (Reserved)
CMD6:	B	SPIN
;---------------------------------
; Command 7. Test (Reserved)
CMD7:	B	COMMAND
;=============================================================================
	.TITLE	"Checksum"
	.PAGE
CHECKSUM:
	LDPK	0		; Data Memory Page 0

	LACK	1
	SACL	one

	ZAC
	SACL	zero
	SACL	chksum

	LAC	one,12
	SUB	one
	SACL	padr		; padr = 4095

	LACK	0FFh
	SACL	bmask		; for masking low bytes

; Read program memory
CKLOOP:	LAC	padr
	TBLR	temp1

; Add to checksum. 
; Separate the bytes so it will be compatible with DIO checksums
	LAC	temp1
	AND	bmask		; 00FF
	ADD	chksum 
	SACL	chksum 		; we have done the low byte

	LAC	temp1,8
	SACH	temp1		; temp1 now contains the high byte in d0-d7
	LAC	temp1
	AND	bmask
	ADD 	chksum 
	SACL	chksum

; Decrement pointer
	ZALS	padr
	SUB	one
	SACL	padr

; Are we done?
	BGEZ	CKLOOP

; Report
	OUT	chksum,POUT
	RET
;----------------------------------------------------------------------
	.TITLE	"Transmit"
	.PAGE

MINIT:	LDPK	0			; Speed WPM = 2.4 x dots/sec
	LARP	AR0

	LACK	1
	SACL	one

	LAC	one,5			
	SACL	speed

	ZAC
	SACL	zero
	SACL	chksum

	SUB	one
	SACL	allon		; FFFF

	RET
;---------------------------
MWAIT:	LAC	one,11
	SACL	cntr1
	ZALS	cntr1
WLOOP:	SUB	one
	BNZ	WLOOP
	RET
;---------------------------
DIT:	LAC	speed
	SACL	cntr2

DLOOP:	OUT	zero,XFLAG
	CALL	MWAIT
	OUT	allon,XFLAG
	CALL	MWAIT
	
	ZALS	cntr2
	SUB	one
	SACL	cntr2
	BNZ	DLOOP
	RET
;---------------------------
DAH:	LT	speed
	MPYK	3
	PAC
	SACL	cntr2
	B	DLOOP
;---------------------------
	.PAGE
ELSPACE:
	LAC	speed			; speed
	SACL	cntr2
ELOOP:	OUT	zero,XFLAG
	CALL	MWAIT
	OUT	zero,XFLAG
	CALL	MWAIT

	ZALS	cntr2
	SUB	one
	SACL	cntr2
	BNZ	ELOOP

	RET
;---------------------------
CHSPACE:
	CALL	ELSPACE	     ; character spaces already have one element space
	CALL	ELSPACE
	CALL	ELSPACE
	RET
;---------------------------
WORDSPACE:
	LARP	AR0		; use AR0 as the counter
	LARK	AR0,8		
WSPLP:	CALL	ELSPACE
	BANZ	WSPLP		; also decrements AR0
	RET
;=============================================================================
XMIT:	LDPK	0			; Speed WPM = 2.4 x dots/sec
	LARP	AR0

	LACK	1
	SACL	one

	LAC	one,5			
	SACL	speed

	ZAC
	SACL	zero
	SACL	chksum

	SUB	one
	SACL	allon			; FFFF

; Setup the counter mask
	LACK	7
	SACL	cmask

; Setup the message counter
	LACK	MSGADR
	TBLR	mptr			; mptr points to the message

; Setup the Code Table
	LACK	CTABLE
	TBLR	ctable

; get the next word pair
XLP1:	LAC	mptr
	TBLR	wpair			; next data

	LAC	wpair,8			; watch out, it is sign extended
	SACH	temp1			; First byte of pair

	CALL	MSEND			; send word in temp1

	LACK	0FFh			; mask
	AND	wpair
	SACL	temp1			; send word in temp1

	CALL	MSEND

; we are done with this pair, increment the message pointer
	ZALS	mptr
	ADD	one
	SACL	mptr
	B	XLP1			; next word
;---------------------------------------------------------------------------
MSEND:	LACK	0FFh
	AND	temp1			; offset of word to send
	ADD	ctable			; Address of Code Table
	TBLR	temp1			; read data at that address

; check if it is a word space
	LAC	temp1
	BNZ	XLP2
	CALL	WORDSPACE
	RET				

; check if it is the end
XLP2:	SUB	allon			; 0FFFF = termination character
	BZ	MOUT			; we are done

; mask off the low 3 bits to be the element counter	
	LAC	temp1
	SUB	one
	AND	cmask
	SACL	temp2
	LAR	AR0,temp2		; AR0 is the element counter

XLP3:	LAC	temp1,1
	SACL	temp1
	BLZ	XLP4			; minus = dah
	CALL	DIT
	B	XLP5

XLP4:	CALL	DAH

XLP5:	CALL	ELSPACE			; element space

XLP6:	BANZ	XLP3	       ; next element in the word, also decrements AR0
	CALL	CHSPACE	       ; character space 

	RET			; We are done with this letter

MOUT:	POP			; We are done with the message
	B XMIT			; Repeat
;=============================================================================
OMSG:	.WORD	0004|0A000h		; C	-.-.	1010	4
	.WORD	0003|0E000h		; O	---	111	3
	.WORD	0004|06000h		; P	.--.	0110	4
	.WORD	0004|0B000h		; Y	-.--	1011	4
	.WORD	0003|04000h		; R	.-.	010	3
	.WORD	0002|00000h		; I	..	00	2
	.WORD	0003|0C000h		; G	--.	110	3
	.WORD	0004|00000h		; H	....	0000	4
	.WORD	0001|08000h		; T	-	1	1
	.WORD	0000			; 			0
	.WORD	0002|4000h		; A	.-	01	2
	.WORD	0001|8000h		; T	-	1	1
	.WORD	0002|4000h		; A	.-	01	2
	.WORD	0003|4000h		; R	.-.	010	3
	.WORD	0002|0000h		; I	..	00	2
	.WORD	0000			; 			0
	.WORD	0003|0C000h		; G	--.	110	3
	.WORD	0002|04000h		; A	.-	01	2
	.WORD	0002|0C000h		; M	--	11	2
	.WORD	0001|00000h		; E	.	0	1
	.WORD	0003|00000h		; S	...	000	3
	.WORD	0000			; 			0
	.WORD	0005|07800h		; 1	.----	01111	5
	.WORD	0005|0F000h		; 9	----.	11110	5
	.WORD	0005|0F000h		; 9	----.	11110	5
	.WORD	0005|0F800h		; 0	-----	11111	5
	.WORD	0FFFFh

CTABLE:	.WORD	0005|0F800h		; 0	-----	11111	5	0
	.WORD	0005|7800h		; 1	.----	01111	5	1
	.WORD	0005|3800h		; 2	..---	00111	5	2
	.WORD	0005|1800h		; 3	...__	00011	5	3
	.WORD	0005|0800h		; 4	....-	00001	5	4
	.WORD	0005|0000h		; 5	.....	00000	5	5
	.WORD	0005|8000h		; 6	-....	10000	5	6
	.WORD	0005|0C00h		; 7	--...	11000	5	7
	.WORD	0005|0E000h		; 8	---..	11100	5	8
	.WORD	0005|0F800h		; 9	----.	11110	5	9

	.WORD	0002|4000h		; A	.-	01	2	10  A
	.WORD	0004|8000h		; B	-...	1000	4	11  B
	.WORD	0004|0A000h		; C	-.-.	1010	4	12  C
	.WORD	0003|8000h		; D	-..	100	3	13  D
	.WORD	0001|0000h		; E	.	0	1	14  E
	.WORD	0004|2000h		; F	..-.	0010	4	15  F
	.WORD	0003|0C000h		; G	--.	110	3	16  10
	.WORD	0004|0000h		; H	....	0000	4	17  11
	.WORD	0002|0000h		; I	..	00	2	18  12
	.WORD	0004|7000h		; J	.---	0111	4	19  13
	.WORD	0003|0A000h		; K	-.-	101	3	20  14
	.WORD	0004|4000h		; L	.-..	0100	4	21  15
	.WORD	0002|0C000h		; M	--	11	2	22  16
	.WORD	0002|8000h		; N	-.	10	2	23  17
	.WORD	0003|0E000h		; O	---	111	3	24  18
	.WORD	0004|6000h		; P	.--.	0110	4	25  19
	.WORD	0004|0D000h		; Q	--.-	1101	4	26  1A
	.WORD	0003|4000h		; R	.-.	010	3	27  1B
	.WORD	0003|0000h		; S	...	000	3	28  1C
	.WORD	0001|8000h		; T	-	1	1	29  1D
	.WORD	0003|2000h		; U	..-	001	3	30  1E
	.WORD	0004|1000h		; V	...-	0001	4	31  1F
	.WORD	0003|6000h		; W	.--	011	3	32  20
	.WORD	0004|9000h		; X	-..-	1001	4	33  21
	.WORD	0004|0B000h		; Y	-.--	1011	4	34  22
	.WORD	0004|0C000h		; Z	--..	1100	4	35  23

	.WORD	0006|7C000h		; .	.-----	011111	6	36  24
	.WORD	0006|0CC00h		; ,	--..--	110011	6	37  25
	.WORD	0006|03000h		; ?	..--..	001100	6	38  26
	.WORD	0005|08100h		; --	-...-	10001	5	39  27

	.WORD	0000|0000h		; SP			0	40  28

MSG:	.WORD	0C18h	; CO
	.WORD	1922h	; PY
	.WORD	2012h	; RI
	.WORD	1011h	; GH
	.WORD	1D28h	; T
	.WORD	0A1Dh	; AT
	.WORD	0A20h	; AR
	.WORD	1228h	; I
	.WORD	100ah	; GA
	.WORD	160Eh	; ME
	.WORD	1C28h	; S
	.WORD	0109h	; 19
	.WORD	0902h	; 92
	.WORD	28FFh	;  
;=============================================================================
	.TITLE	"RAM Test"
	.PAGE
RAMTEST:
;{---------------------------------------------------------------}
	LACK	0	;{ Fill memory with '0'}
	CALL	FILL
;{---------------------------------------------------------------}
;{ Start with memory set to zero,
;  Start with Address 0, check that the data is '0', 
;  Turn on each bit, (D0-D15), check the word, Do the next bit,
;  Do the next word forward. 
;  Ends with memory set to FFFF }
	CALL	RTST1
;{---------------------------------------------------------------}
;{ Start with memory set to 'FFFF',
;  Start with Address 0, check that the data is 'FFFF', 
;  Turn off each bit, (D0-D15), check the word, 
;  Do the next bit,
;  Do the next word forward. 
;  Ends with memory set to '0' }
	CALL	RTST2		
;{---------------------------------------------------------------}
;{ Start with memory set to zero,
;  Start with Address 254, check that the data is '0', 
;  Turn on each bit, (D15-D0), check the word, Do the next bit,
;  Do the next word backward. 
;  End with memory set to 'FFFF' }
	CALL	RTST3		
;{---------------------------------------------------------------}
;{ Start with memory set to 'FFFF',
;  Start with Address 8191, check that the data is 'FFFF', 
;  Turn off each bit, (D15-D0), check the word, Do the next bit,
;  Do the next word backward.
;  Ends with memory set to '0' }
	CALL	RTST4		
;{---------------------------------------------------------------}
;{ Start with memory set to '0',
;  Start with Address 0, check that the data is '0', 
;  Turn each bit (D15-D0) on, check the word, then off, check the word, 
;  Do the next bit, Do the next word forward. 
;  End with memory set to '0' }
	CALL	RTST5;		
;{---------------------------------------------------------------}
;{ Start with memory set to '0',
;  Start with Address 8191, check that the data is '0', 
;  Turn each bit (D0 - D15) on, check the word, then off, check the word, 
;  Do the next bit, Do the next word backward. 
;  Ends with memory set to '0' }
	CALL	RTST6				
;{---------------------------------------------------------------}
;{ Fill memory with 'FFFF'}
	LACK	1
	SACL	temp
	LACK	0
	SUB	temp
	CALL	FILL
;{---------------------------------------------------------------}
;{ Start with memory set to 'FFFF',
;  Start with Address 0, check that the data is 'FFFF', 
;  Turn each bit (D15 - D0) off, check the word, then on, check the word, 
;  Do the next bit, Do the next word forward. 
;  Ends with memory set to 'FFFF'.}
	CALL	RTST7		
;{---------------------------------------------------------------}
;{ Start with memory set to 'FFFF',
;  Start with Address 8191, check that the data is 'FFFF', 
;  Turn each bit (D15 - D0) off, check the word, then on, check the word, 
;  Do the next bit, Do the next word backward. 
;  Ends with memory set to 'FFFF' }
	CALL	RTST8		
;{---------------------------------------------------------------}
;{ Fill memory with Data = Address
;  Read memory with Data = Address }
	CALL	RTST9		
;{---------------------------------------------------------------}
;{ Fill memory with Data = 5555,
;  Start with Address 0, check that the data is 5555,
;  Change data to AAAA, check the data,
;  Do the next word forward. }
	CALL	RTST10		
;{---------------------------------------------------------------}
; It got here so it must have been successful
	LACK	0		
	SACL	temp
	OUT	temp,POUT

	B	SPIN
;{---------------------------------------------------------------}
FILL:	LARP	AR0
	LARK	AR0,255
FILLP:	SACL	*		; @AR1
	BANZ	FILLP
	RET
;{---------------------------------------------------------------}
	.PAGE
;{ Start with memory set to zero,
;  Start with Address 0, check that the data is '0', 
;  Turn on each bit, (D0-D15), check the word, Do the next bit,
;  Do the next word forward. 
;  Ends with memory set to FFFF }
RTST1:	LDPK	1		
	LARK	AR1,0		; Address Counter
	LARP	AR1
R1LP1:	LACK	0		; for the error routine (just in case)
	SACL	temp

	LAC	*
	BNZ	ERR1		; should start out being '0'
	LACK	1
	SACL	temp
	LARK	AR0,15		; Bit Counter
	LAC	temp
R1LP2:	LARP	AR1
	SACL	*
	SUBS	*
	BNZ	ERR1
	LAC	temp,1
	OR	temp
	SACL	temp	
	LARP	AR0
	BANZ	R1LP2		; also decrements AR0

; next address
	LACK	1
	LARP	AR1
	MAR	*+		; increment address

	SAR	AR1,temp
	LACK	255
	SUBS	temp
	BNZ	R1LP1		; next word

	RET			; All Done
;{---------------------------------------------------------------}
	.PAGE
;{ Start with memory set to 'FFFF',
;  Start with Address 0, check that the data is 'FFFF', 
;  Turn off each bit, (D0-D15), check the word, 
;  Do the next bit,
;  Do the next word forward. 
;  Ends with memory set to '0' }
RTST2:	LDPK	1		
	LARK	AR1,0		; Address Counter

R2LP1:	LACK	1		; get FFFF into temp
	SACL	temp
	LACK	0
	SUB	temp
	SACL	temp

	LARP	AR1
	ZALS	*		; is it FFFF?
	SUBS	temp
	BNZ	ERR2		; should start out being 'FFFF'

	LARK	AR0,15		; Bit Counter

	LAC	temp,1		; shift zeros into d0
	SACL	temp	

R2LP2:	LARP	AR1

	LAC	temp
	SACL	*

	ZALS	*		
	SUBS	temp
	BNZ	ERR2		

	LAC	temp,1		; shift zeros into d0
	SACL	temp	

	LARP	AR0		; bit counter
	BANZ	R2LP2		; also decrements AR0

; next address
	LARP	AR1
	MAR	*+		; increment address

	SAR	AR1,temp
	LACK	255
	SUBS	temp
	BNZ	R2LP1		; next word

	RET			; All Done
;{---------------------------------------------------------------}
	.PAGE
;{ Start with memory set to zero,
;  Start with Address 254, check that the data is '0', 
;  Turn on each bit, (D15-D0), check the word, Do the next bit,
;  Do the next word backward. 
;  End with memory set to 'FFFF' }
RTST3:	LDPK	1		
	LARK	AR1,254		; Address Counter

R3LP1:	LARP	AR1

	LACK	0		; for the error routine (just in case)
	SACL	temp

	ZALS	*		; is it 0000?
	BNZ	ERR3		; should start out being '0000'

	LARK	AR0,15		; Bit Counter

	LACK	1		; set temp = 8000
	SACL	temp	
	LAC	temp,15
	SACL	temp

R3LP2:	LARP	AR1

	LAC	temp
	SACL	*

	ZALS	*		
	SUBS	temp
	BNZ	ERR3		

	LAC	temp,15		; shift ones into d15
	SACH	temp		; Sign Extend makes it work

	LARP	AR0		; bit counter
	BANZ	R3LP2		; also decrements AR0

; next address
	LARP	AR1
	BANZ	R3LP1		; next word

	RET			; All Done
;{---------------------------------------------------------------}
	.PAGE
;{ Start with memory set to 'FFFF',
;  Start with Address 8191, check that the data is 'FFFF', 
;  Turn off each bit, (D15-D0), check the word, Do the next bit,
;  Do the next word backward.
;  Ends with memory set to '0' }

RTST4:	LDPK	1		
	LARK	AR1,254		; Address Counter

R4LP1:	LACK	1		; get FFFF into temp
	SACL	temp
	LACK	0
	SUB	temp
	SACL	temp

	LARP	AR1
	ZALS	*		; is it FFFF?
	SUBS	temp
	BNZ	ERR4		; should start out being 'FFFF'

	LARK	AR0,15		; Bit Counter

	LACK	1		; set temp = 7FFF
	SACL	temp	
	LAC	temp,15		; temp = 8000
	SUB	temp
	SACL	temp		; temp = 7FFF

R4LP2:	LARP	AR1

	LAC	temp
	SACL	*

	ZALS	*		
	SUBS	temp
	BNZ	ERR4		

	LAC	temp,15		; shift zeroes into d15
	SACH	temp		; Sign Extend makes it work

	LARP	AR0		; bit counter
	BANZ	R4LP2		; also decrements AR0

; next address
	LARP	AR1
	BANZ	R4LP1		; next word

	RET			; All Done
;{---------------------------------------------------------------}
	.PAGE
;{ Start with memory set to '0',
;  Start with Address 0, check that the data is '0', 
;  Turn each bit (D15-D0) on, check the word, then off, check the word, 
;  Do the next bit, Do the next word forward. 
;  End with memory set to '0' }
RTST5:	LDPK	1		
	LARK	AR1,0		; Address Counter
	LARP	AR1

R5LP1:	LACK	0		; for the error routine (just in case)
	SACL	temp

	LAC	*
	BNZ	ERR5		; should start out being '0'

R5LP2:	LACK	1
	SACL	temp
	LAC	temp,15		; temp = 8000, will be done separately

	LARP	AR1

	SACL	*
	SUBS	*
	BNZ	ERR5

	LACK	0
	SACL	*
	SUBS	*
	BNZ	ERR5

	LACK	1
	SACL	temp
	LAC	temp,14		
	SACL	temp		; temp = 4000

	LARK	AR0,14		; Bit Counter
R5LP3:	LARP	AR1
	LAC	temp
	SACL	*
	SUBS	*
	BNZ	ERR5

	LACK	0
	SACL	*
	SUBS	*
	BNZ	ERR5

	LAC	temp,15
	SACH	temp

	LARP	AR0
	BANZ	R5LP3		; also decrements AR0

; next address
	LACK	1
	LARP	AR1
	MAR	*+		; increment address

	SAR	AR1,temp
	LACK	255
	SUBS	temp
	BNZ	R5LP1		; next word

	RET			; All Done
;{---------------------------------------------------------------}
	.PAGE
;{ Start with memory set to '0',
;  Start with Address 254, check that the data is '0', 
;  Turn each bit (D0 - D15) on, check the word, then off, check the word, 
;  Do the next bit, Do the next word backward. 
;  Ends with memory set to '0' }
RTST6:	LDPK	1		
	LARK	AR1,254		; Address Counter
	LARP	AR1

R6LP1:	LACK	0		; for the error routine (just in case)
	SACL	temp

	LAC	*
	BNZ	ERR6		; should start out being '0'

R6LP2:	LACK	1
	SACL	temp

	LARK	AR0,15		; Bit Counter
R6LP3:	LARP	AR1
	ZALS	temp
	SACL	*
	SUBS	*
	BNZ	ERR6

	LACK	0
	SACL	*
	SUBS	*
	BNZ	ERR6

	LAC	temp,1
	SACL	temp

	LARP	AR0
	BANZ	R6LP3		; also decrements AR0

; next address
	LARP	AR1

	BANZ	R6LP1		; test for AR1=0 and decrement
	RET			; All Done
;{---------------------------------------------------------------}
	.PAGE
;{ Start with memory set to 'FFFF',
;  Start with Address 0, check that the data is 'FFFF', 
;  Turn each bit (D15 - D0) off, check the word, then on, check the word, 
;  Do the next bit, Do the next word forward. 
;  Ends with memory set to 'FFFF'.}
;  Requires two RAM locations to make it work
RTST7:	LDPK	1		
	LARK	AR1,0		; Address Counter

	LACK	1		; this test requires an additional RAM location
	SACL	xtemp
	LACK	0
	SUB	xtemp
	SACL	xtemp

; Do 7FFF separately
R7LP1:	LACK	1		; get FFFF into temp
	SACL	temp
	LACK	0
	SUB	temp
	SACL	temp

	LARP	AR1
	ZALS	*		; is it FFFF?
	SUBS	temp
	BNZ	ERR7		; should start out being 'FFFF'

	LACK	1		; set temp = 7FFF
	SACL	temp	
	LAC	temp,15		; temp = 8000
	SUB	temp
	SACL	temp		; temp = 7FFF

	LARP	AR1

	LAC	temp		; turn the bit off
	SACL	*

	ZALS	*		; test the bit
	SUBS	temp
	BNZ	ERR7		

	ZALS	xtemp		; turn it on
	SACL	*

	SUBS	*		; test it
	BNZ	ERR7

	LACK	1		; set up BFFF
	SACL	temp	
	LAC	temp,14		; ACC = 4000
	ADD	temp		; ACC = 4001
	SACL	temp		
	LACK	0
	SUB	temp		; ACC = BFFF = (0 - 4001)
	SACL	temp		; temp = BFFF

	LARK	AR0,14		; Bit Counter

R7LP2:	LARP	AR1

	LAC	temp		; turn the bit off
	SACL	*

	ZALS	*		; test the bit
	SUBS	temp
	BNZ	ERR7		

	ZALS	xtemp		; turn it on
	SACL	*

	SUBS	*		; test it
	BNZ	ERR7

	LAC	temp,15		; rotate right (sign extends)
	SACH	temp		

	LARP	AR0		; bit counter
	BANZ	R7LP2		; also decrements AR0

; next address
	LACK	1
	LARP	AR1
	MAR	*+		; increment address

	SAR	AR1,temp
	LACK	254
	SUBS	temp
	BNZ	R7LP1		; next word

	RET			; All Done
;{---------------------------------------------------------------}
	.PAGE
;{ Start with memory set to 'FFFF',
;  Start with Address 253, check that the data is 'FFFF', 
;  Turn each bit (D15 - D0) off, check the word, then on, check the word, 
;  Do the next bit, Do the next word backward. 
;  Ends with memory set to 'FFFF' }
RTST8:	LDPK	1		
	LARK	AR1,253		; Address Counter

	LACK	1		; this test requires an additional RAM location
	SACL	xtemp
	LACK	0
	SUB	xtemp
	SACL	xtemp		; xtemp = FFFF

; Do 7FFF separately
R8LP1:	LACK	1		; get FFFF into temp
	SACL	temp
	LACK	0
	SUB	temp
	SACL	temp

	LARP	AR1
	ZALS	*		; is it FFFF?
	SUBS	temp
	BNZ	ERR8		; should start out being 'FFFF'

	LACK	1		; set temp = 7FFF
	SACL	temp	
	LAC	temp,15		; temp = 8000
	SUB	temp
	SACL	temp		; temp = 7FFF

	LAC	temp		; turn the bit off
	SACL	*

	ZALS	*		; test the bit
	SUBS	temp
	BNZ	ERR8		

	ZALS	xtemp		; turn it on
	SACL	*

	SUBS	*		; test it
	BNZ	ERR8

	LACK	1		; set up BFFF
	SACL	temp	
	LAC	temp,14		; ACC = 4000
	ADD	temp		; ACC = 4001
	SACL	temp		
	LACK	0
	SUB	temp		; ACC = BFFF = (0 - 4001)
	SACL	temp		; temp = BFFF

	LARK	AR0,14		; Bit Counter

R8LP2:	LARP	AR1

	LAC	temp		; turn the bit off
	SACL	*

	ZALS	*		; test the bit
	SUBS	temp
	BNZ	ERR8		

	ZALS	xtemp		; turn it on
	SACL	*

	SUBS	*		; test it
	BNZ	ERR8

	LAC	temp,15		; rotate right (sign extends)
	SACH	temp		

	LARP	AR0		; bit counter
	BANZ	R8LP2		; also decrements AR0

; next address
	LARP	AR1
	BANZ	R8LP1		; check loop counter and decrement

	RET			; All Done
;{---------------------------------------------------------------}
	.PAGE
;{ Fill memory with Data = Address
;  Read memory with Data = Address }
RTST9:	LARP	AR1
	LARK	AR1,254		; Address Counter
R9LP1:	SAR	AR1,temp
	LAC	temp
	SACL	*
	BANZ	R9LP1

	LARK	AR1,254
R9LP2:	SAR	AR1,temp
	ZALS	*
	SUBS	temp
	BNZ	ERR9
	BANZ	R9LP2

	RET	
;{---------------------------------------------------------------}
;{ Fill memory with Data = 5555,
;  Start with Address 253, check that the data is 5555,
;  Change data to AAAA, check the data,
;  Do the next word backward. }
RTST10:	LARP	AR1


	LACK	0AAh
	SACL	xtemp
	LAC	xtemp,8
	OR	xtemp
	SACL	xtemp		; xtemp = AAAA

	LACK	55h
	SACL	temp
	LAC	temp,8
	OR	temp
	SACL	temp		; temp = 5555

	LARK	AR1,253		; Address Counter
R10LP1:	SACL	*
	BANZ	R10LP1		; fill memory with 5555

	LARK	AR1,253
R10LP2:	ZALS	*		; is it 5555?
	SUBS	temp
	BNZ	ERR10

	ZALS	xtemp		; change it to AAAA
	SACL	*
	SUBS	*		; did it change?
	BNZ	ERR10		

	BANZ	R10LP2

	RET		
;{---------------------------------------------------------------}
	.PAGE
ERR1:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	1h
	SACL	temp1		; tst #
	B	ERROUT

ERR2:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	2h
	SACL	temp1		; tst #
	B	ERROUT

ERR3:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	3h
	SACL	temp1		; tst #
	B	ERROUT

ERR4:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	4h
	SACL	temp1		; tst #
	B	ERROUT

ERR5:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	5h
	SACL	temp1		; tst #
	B	ERROUT

ERR6:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	6h
	SACL	temp1		; tst #
	B	ERROUT

ERR7:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	7h
	SACL	temp1		; tst #
	B	ERROUT

ERR8:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	8h
	SACL	temp1		; tst #
	B	ERROUT

ERR9:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	9h
	SACL	temp1		; tst #
	B	ERROUT

ERR10:	LAC	temp		; data should have been
	LDPK	0
	SACL	temp2

	LACK	10h
	SACL	temp1		; tst #
	B	ERROUT
;-------------------------------------------------------------------------
	.PAGE
ERROUT:	LAC	*		; data was
	SACL	temp3

	SAR	AR1,temp4	; address at which error occurred

	LACK	0FFh		; send the error flag
	CALL	SEND

	LAC	temp1		; test #
	CALL	SEND	

	LAC	temp4		; address
	CALL	SEND	

	LAC	temp2		; data should be
	CALL	SEND

	LAC	temp3		; data was
	CALL	SEND

ERRLP:	B	SPIN
;{---------------------------------------------------------------}
; Send value in Accumulator to Output Port
SEND:	SACL	mptr
	OUT	mptr,POUT

SLP:	IN	status,STAT
	LAC	status
	BLZ	SLP		; wait for the 68010 to read the port 
	RET
;{---------------------------------------------------------------}
; Test the Error routine
ERRTST:	LDPK	1		; Data Page 1
	LARK	AR1,0A5h	; Address Counter
	LARP	AR1

	LACK	23h
	SACL	*		; Data was

	LACK	5Ah
	SACL	temp		; Data should have been

	B	ERR10
;{---------------------------------------------------------------}
	.END
