
	TTL 'CONDITS'

* SAMPLES FOR CONDITIONAL ASSEMBLY OPERATORS

	INCLUDE 'COMN.ASM'

	SECTION	PROGRAM
	PAGE
* Look at (and rotate) individual objects

	XDEF RTBL,LCUBE,LTANK,LROC,LAIR,LSAU,LHIND
	XDEF LRESU,LMLOOP,DO_OBJ

RED	EQU	4
GREEN	EQU	2
BLUE	EQU	1

	XREF CUBE,SQ,TANK,ROCKET,AIR,SAUCER,BUZZ,HIND,CITY,VJ_BKBOX
	XREF D_CUBE,D_SQ,D_TANK,D_ROC,D_AIR,D_SAU,D_BUZZ,D_HIND,D_CITY
	XREF NT_ROLL,NT_PTCH,NT_YAW,NWC_GEN,NCONCAT,NPOST
	XREF INITB,VWAIT

CSTART:
RTBL:	
	DC.L	CUBE,D_CUBE,8
	DC.B	RED,$40

	DC.L	ROCKET,D_ROC,25
	DC.B	BLUE,$40

	DC.L	TANK,D_TANK,24
	DC.B	GREEN,$40

	DC.L	AIR,D_AIR,24
	DC.B	BLUE,$40

	DC.L	SAUCER,D_SAU,17
	DC.B	GREEN,$40

	DC.L	HIND,D_HIND,54
	DC.B	RED,$40

* Rotate the Objects to see what they look like, using only 68010

LCUBE:	MOVE.W	#0,D0
	BRA	LRESU
LROC:	MOVE.W	#1,D0
	BRA	LRESU
LTANK:	MOVE.W	#2,D0
	BRA	LRESU
LAIR:	MOVE.W	#3,D0
	BRA	LRESU
LSAU:	MOVE.W	#4,D0
	BRA	LRESU
LHIND:	MOVE.W	#5,D0
	BRA	LRESU

LRESU:	MOVE.L	#USTACK,A7	*Initialise stack pointer
	MOVE.W 	#$2F00,SR	*Disable Interrupts

* Convert Model Number into Table Address
	MULU	#14,D0		
	LEA.L	RTBL,A0
	ADDA.W	D0,A0
	MOVE.L	A0,MODEL
* Initialise Unit Vectors, Reset VG, and Upload Alpha-Numerics 
	JSR	INITB		

*Main Loop
LMLOOP	
	MOVEA.L	#VMEM,A0
	MOVE.W	#$8040,(A0)+		*CENTER
	MOVE.W	#VJ_BKBOX,(A0)+		*Beam Killer Box
	MOVE.L	A0,VG_PTR		*Save the VG Pointer

	MOVEA.W	#CCUV,A0	*Concatenate Unit Vectors
	MOVEA.W	#UV2,A2		*UV2 is the working copy (Ship)
	MOVEA.W	#UV3,A1		*UV3 is currently the identity matrix (Observer)
	JSR	NCONCAT		*Ship in A1 looks at Ship in A2 (Naked)

	MOVEA.W	#CCUV,A0	*Translate Ship
	MOVE.W	#$100,XT(A0)	
	CLR.W	YT(A0)
	CLR.W	ZT(A0)

	BSR	DO_OBJ		*Do the selected object

	MOVE.W	#$2020,(A0)	*VG Halt Instruction
	CLR.W	VGGO		*Start VG	

	BSR 	POTS		*Read and mediate the Pots

	BSR	TROTATE		*Rotate as requested by the Pots and Switches

*Generate the Working Copy Unit Vectors for Ship 1
	MOVE.W	#UV2,A0
	MOVE.W	#UV1,A1
	JSR 	NWC_GEN		* Generate the Working Copy (Naked)

	BSR 	VWAIT		*Wait for VG to be done	

	BRA	LMLOOP

*---
DO_OBJ:
	MOVE.L	MODEL,A3	* Model Select Table Data, 

	MOVEA.W #CCUV,A0	*Post Add Transform CCUV to Point Table
	MOVE.L	(A3),A1		* 0(A1) is address of Object Picture (ex. CUBE)
	MOVEA.W	#PTBL,A2	* Put Projected points in the Point Table
	MOVE.L	8(A3),D5	* Number of Points (ex. '8')
	JSR	NPOST		* Post Add and Project

	MOVE.L	MODEL,A3
	MOVE.B	12(A3),COLOR
	MOVE.B	13(A3),BRITE
	MOVE.L	4(A3),A3
	JSR	(A3)		* Drawing Routine (ex. 'D_CUBE')
	RTS
*===============================================================================
* ROTATIONS FOR TEST PROGRAM
* Takes the values in POT0 and POT1 and reads Control Switches to
* Rotate the Ship.
* Takes about 440 us for three axes (for each ship)
TROTATE:
*Ship 1
*ROLL TIC
* D0 CONTAINS DELTA ANGLE FOR YAW, +/- 79 MAX
	MOVE.B	POT1,D0			*Yaw Pot
	MOVEA.W	#UV1,A0			*A0=Ship 1 primary
	JSR	NT_YAW			* Naked Tic_Yaw

*PITCH TIC
* D0 CONTAINS DELTA ANGLE FOR PITCH, +/- 79 MAX
	MOVE.B	POT0,D0			*Pitch Pot
	MOVEA.W	#UV1,A0			*A0=Ship 1 primary
	JSR	NT_PTCH			* Naked Tic_Pitch

*Read Trigger switches for ROLL
	MOVEA.W	#UV1,A0			*A0=Ship 1 primary

	BTST	#04,INPUTS
	BNE	RO1
	MOVE.B	#$10,D0
	JSR	NT_ROLL			* Naked Tic Roll
	RTS

RO1	BTST	#05,INPUTS
	BNE	RO2
	MOVE.B	#-$10,D0
	JSR	NT_ROLL			* Naked Tic Roll

RO2	RTS

POTS:
* READ THE POTS
	MOVE.B	#$0,ADCON+1		*CHANNEL 0
	MOVE.B	#$8,ADCON+1		*ALE HIGH
	MOVE.B	#10,D0			*WAIT A LITTLE
P10	SUBQ.B	#1,D0
	BNE	P10
	MOVE.B	#$0,ADCON+1		*ALE LOW

	MOVE.B	#-40,D0
P20	ADDQ.B	#1,D0			*WAIT
	BNE 	P20	

	MOVE.B	ADCREAD+1,RPOT0		*READ AND STORE POT 0

	MOVE.B	#1,ADCON+1		*CHANNEL 1
	MOVE.B	#$9,ADCON+1		*ALE HIGH
	MOVE.B	#10,D0			*WAIT A LITTLE
P30	SUBQ.B	#1,D0
	BNE	P30
	MOVE.B	#1,ADCON+1		*ALE LOW

	MOVE.B	#-40,D0
P40	ADDQ.B	#1,D0			*WAIT
	BNE 	P40	

	MOVE.B	ADCREAD+1,RPOT1		*READ AND STORE POT 1

* Mediate the Pots
	MOVE.B	RPOT0,D0
	SUB.B	#$80,D0
	ASR.B	#2,D0
	MOVE.B	D0,POT0

	MOVE.B	RPOT1,D0
	NOT.B	D0
	SUB.B	#$80,D0
	ASR.B	#2,D0
	MOVE.B	D0,POT1

	RTS
	

CEND
	TTL 'CONDITIONAL EXAMPLES'
	PAGE

	IFEQ	0
	FAIL	1000
	ENDC


	IFEQ	0
	FAIL	'IFEQ 0'
	ENDC


	IFGT	0
	FAIL	'IFGT 0'
	ENDC


	IFGE	0
	FAIL	'IFGE 0'
	ENDC


	IFGT	3
	FAIL	'IFGT 3'
	ENDC


	IFGT	CEND-CSTART-$100
	FAIL	'CEND-CSTART .GT. $100'
	ENDC


	IFGT	CEND-CSTART-$400
	FAIL	'CEND-CSTART .GT. $400'
	ENDC


	FAIL	CEND			* Doesn't work (Not Absolute Expression)


	FAIL	CEND-CSTART	


	FAIL	'CEND-CSTART=',CEND-CSTART	* Doesn't work


	FAIL	'CEND=';CEND-CSTART		* Doesn't work


	FAIL	'CEND-CSTART = '
	FAIL	CEND-CSTART

	WARN	'The Weed of Crime Bears Bitter Fruits'  	* Doesn't Work
	WARNING	'The Weed of Crime Bears Bitter Fruits'  	* Doesn't Work

TEMP	EQU	$100
	FAIL	TEMP	

* Alphas in MACROs?

TEST	SET	0

MACTST	MACRO
	DC.B	\1
	ENDM

	MACTST	TEST
	MACTST	'TEST'

ALPHA	SET	'12345678'
	DC.B	ALPHA

	DC.B	(ALPHA)&($0FF)

	DC.B	(ALPHA/256)&($0FF)

	DC.B	(ALPHA/65536)&($0FF)

	DC.B	(ALPHA/65536/256)&($0FF)

	DC.B	(ALPHA/65536/65536)&($0FF)

	DC.B	(ALPHA/65536/65536/256)&($0FF)

	END
