	TTL 'OLD VGMC'

* For Reference only

	INCLUDE 'EQUATES.ASM'
	XDEF	VJ_BKBOX

* TomCat VG Macros
* Original Macros programmed by ED LOGG, 8 October 1979
* Updated/Revised/Enhanced for Star Wars in June '83, by GREGR and NORM A.
* Rejuberated for ASM68 by: Jed Margolin, April 1985.

*---
*	CNTR - Center the beam in the middle of the screen.
*---
CNTR	MACRO 
	DC.W	$8040			*VGCNTR
	ENDM
*---
*	HALT - When executed by the Vector Generator sets the VG Halt Flag
*---
HALT	MACRO	
	DC.W	$2000
	ENDM
*---
*	JSRL - JUMP SUBROUTINE INSTRUCTION
*	The lower thirteen bits specify a word address relative to the 
*	beginning of the Vector Generator addresable instruction space
*	(relative byte divided by 2). The total address area is 16K bytes
*	Only three levels deep work. The forth overlays the first, and so on.
*
*	This will be a VG memory image residing in 68010 program memory
*	at absolute origin VGIMAGE. 
*
*	The address offset of the Alphanumerics in the Vector Generator address
*	space will be called VGANLOC. If, for example, alphanumerics go in 
*	the last the 4K of VG's 16K address then VGANLOC = $3000  .
* 	For a subroutine at label CHAR_ADD, the JSRL address will be:
*		CHAR_ADD-VGIMAGE+VGANLOC

*---	
JSRL	MACRO
	DC.W	((\1-VGIMAGE+VGANLOC)/2)&$1FFF!$0A000
	ENDM
*---
* 	STAT sets the three color bits (RGB) and the Linear Intensity (0 to FF)
* 	80 is the highest normal intensity; 81 thru FF produce gun saturation
* 	which results in beam spreading and defocussing for special effects.
*
* 	Format: STAT COLOR,ZSCALE
*---
STAT	MACRO
	DC.W	$6000!((\1&$7)*$100)!(\2&$0FF)
	ENDM
*---
*	SCAL - SET VECTOR GENERATOR SCALE
*	The first parameter is the power of 2 scale. A value of 2 means
*	full size, 3=1/2 size, 4=1/4 size, ...,7=1/32 size. The second
*	parameter is the Linear Scale (Optional). A value of 0 means
*	full size, 80=1/2 size, FF=1/256 size. Linear scaling does not
*	change the vector drawing time, it just makes it smaller (and
*	to some extent, brighter).
*
* 	Format:	SCAL BIN,LIN
*---
SCAL	MACRO
	DC.W	$7000!((\1&$7)*$100)!(\2&$0FF)
	ENDM
*---
*	RTSL - RETURN FROM SUBROUTINE 
*	This instruction is analagous to an RTS instruction for a
*	microprocessor. The VG is capable of only three levels of
*	subroutine calls (JSRL).
*---
RTSL	MACRO	
	DC.W	$0C000
	ENDM
*---
*	JMPL - JUMP TO ROUTINE
*	The lower thirteen bits reference an address the same as in
*	a JSRL instruction. There is no return address saved on the
*	VG stack. Refer to JSRL for an explanation of the addressing.
*---
JMPL	MACRO
	DC.W	((\1-VGIMAGE+VGANLOC)/2)&$1FFF!$0E000
	ENDM
*---
*	VCTR - DRAW VECTOR INSTRUCTION
*	This instruction draws a vector on the display relative to
*	the previous beam position. DX is the change in Beam X
*	position, DY is the change in Beam Y position, ZZ specifies
*	the beam intensity (0 thru 7; 0 is no intensity, 7 is brightest
*	intensity). This intensity is scaled by ZSCALE set by the STAT
*	instruction.
*---
*	Short Vectors are drawn twice the length of the unadjusted dx and
*	dy values embedded in the SVECT instruction. Therefore, the
*	Macro will take the DX, DY values in the argument and divide
*	them by two. When they are executed by the hardware they will
*	be doubled back to normal size.
*
*	Since there are 5 bits, the maximum unadjusted range is -16 to +15.
*	With scaling, this becomes -32 to +30, with even values only.
*	A vector that does not meet these requirements will be drawn as a
*	standard (long) vector. Also, short vectors may not have both
*	DX and DY be zero.
*
*	Format:	 VCTR DX,DY,ZZ
*---
VCTR	MACRO
DX	SET	\1	*For temporary storage
DY	SET	\2	*For temporary storage
	IFLT DX		*If DX is negative, make it positive and subtract 2
DX	  SET (-\1)-2	*so that -32 will become +30 instead of +32 which
	ENDC		*wouldn't fit.
	IFLT DY		*Same for DY
DY	  SET (-\2)-2
	ENDC
DXY	SET DX!DY	*OR the positive values together
	IFNE (DX+DY)	*If the result is 0, fall through, become a Long Vector
	  IFEQ DXY&$0FFE1 *If .LT. -32 or .GT. 32 or odd, become a Long Vector
	    DC.B  $40!((\2/2)&$1F),((\3&7)*$20)!((\1/2)&$1F)
	    MEXIT
	  ENDC
	ENDC
	DC.W 	\2&$1FFF,((\3&7)*$2000)!(\1&$1FFF) *Long Vector
	ENDM
*---
*	ALPHA - Constructs a JSRL instruction for every character
*	in the calling list. A Label 'CHAR_X' must exist for every
*	unique character 'X' in the calling list. The list must end
*	with a '~'. Only eight characters are allowed per line (in
*	addition to the delimiter).
*
*	Format:	ALPHA	P,L,A,Y,E,R,~	(example)
*---
ALPHA	MACRO
	IFEQ	'\1'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\1

	IFEQ	'\2'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\2

	IFEQ	'\3'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\3

	IFEQ	'\4'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\4

	IFEQ	'\5'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\5

	IFEQ	'\6'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\6

	IFEQ	'\7'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\7

	IFEQ	'\8'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\8

	IFEQ	'\9'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\9

	ENDM
*---
*	ALPHAV - Same as APLHA, but places the characters vertically from top 
*	to bottom. See ALPHA for more info.
*
*	Format:	ALPHAV	P,L,A,Y,E,R,~	(example)
*---
ALPHAV	MACRO
	IFEQ	'\1'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\1
	VCTR	-24,-36,0

	IFEQ	'\2'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\2
	VCTR	-24,-36,0

	IFEQ	'\3'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\3
	VCTR	-24,-36,0

	IFEQ	'\4'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\4
	VCTR	-24,-36,0

	IFEQ	'\5'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\5
	VCTR	-24,-36,0

	IFEQ	'\6'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\6
	VCTR	-24,-36,0

	IFEQ	'\7'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\7
	VCTR	-24,-36,0

	IFEQ	'\8'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\8
	VCTR	-24,-36,0

	IFEQ	'\9'-'~'
	MEXIT
	ENDC	
	JSRL	CHAR_\9
	VCTR	-24,-36,0

	ENDM
*---
VGANLOC	EQU	$3000	*VG Alphanumerics go in last 4K (of VG's 16K space)	
VGIMAGE EQU	$5000	*Image will be ORG'ed at $5000 for now

*Test Data
RED	EQU	$4
GREEN	EQU	$2
BLUE	EQU	$1
WHT	EQU	$7
VG_BASE	EQU	$3000
OFF	EQU	$00
ON	EQU	$07

	ORG	VGIMAGE

	
*----------------------------------------------------------------------
VJ	MACRO
VJ_\1	EQU	((\1-VGIMAGE+VGANLOC)/2)&$1FFF!$0A000
	ENDM


*Box to fool the beam killer
BKBOX	VJ	BKBOX	
ASP	SET	75
	SCAL	2,0			*Bin=2,Lin=0
	STAT	1,$20			*Color=Blu,Zscale=20
	VCTR	512,512*ASP/100,OFF
	VCTR	-1024,0*ASP/100,ON
	VCTR	0,-1024*ASP/100,ON
	VCTR	1024,0*ASP/100,ON
	VCTR	0,1024*ASP/100,ON
	CNTR
	RTSL

	END

*----------------------------------------------------------------------
	ALPHA	A,L,T,I,T,U,D,E,~

	ALPHA	O,N,E,~

	ALPHA	B,L,,A,N,K,~

	ALPHA	B,L,_,A,N,K,~

	ALPHAV	A,L,T,I,T,U,D,E,~

	ALPHAV	O,N,E,~

	CNTR

	HALT

	CNTR

	CNTR

	HALT

	HALT	

	JSRL	CHAR

	JSRL	CHAR2

	JSRL	CHAR3

	JSRL	CHAR4

	JSRL	CHAR5

	JSRL	CHAR6

	JSRL	CHAR7

	JSRL	CHAR8

	JSRL	CHAR9

	JSRL	CHAR10
*---

	STAT	BLUE,$20

	STAT	GREEN,$20

	STAT	RED,$20

	STAT	WHT,$080

	SCAL	2,$0

	SCAL 	1,$80

	SCAL	4,$20

	RTSL

	RTSL

	JMPL	CHAR

	JMPL	CHAR9

	JMPL	CHAR10
	
	VCTR	100,100,7

	VCTR	-200,0,7

	VCTR	100,-100,4

	VCTR	$100,$100,7

	VCTR	-$200,$0,7

	VCTR	$100,-$100,4

	VCTR	$100,$100,7

	VCTR	-$100,$0,7

	VCTR	$100,-$100,4

	VCTR	$10,$10,7

	VCTR	-$10,$0,7

	VCTR	$10,-$10,7

	VCTR	30,30,1

	VCTR	30,32,1

	VCTR	32,30,1

	VCTR	32,32,1

	VCTR	32,31,1

	VCTR	-32,-32,1

	VCTR	-32,-34,1

	VCTR	-34,-32,1

	VCTR	-34,-34,1

	VCTR	-34,-33,1

	VCTR	30,31,1

	VCTR	31,30,1

	VCTR	31,31,1

	VCTR	0,0,1


CHAR	DC.W	$2020
CHAR_A	DC.W	$2020
CHAR_B	DC.W	$2020
CHAR_C	DC.W	$2020
CHAR_D	DC.W	$2020
CHAR_E	DC.W	$2020
CHAR_F	DC.W	$2020
CHAR_G	DC.W	$2020
CHAR_H	DC.W	$2020
CHAR_I	DC.W	$2020
CHAR_J	DC.W	$2020
CHAR_K	DC.W	$2020
CHAR_L	DC.W	$2020
CHAR_M	DC.W	$2020
CHAR_N	DC.W	$2020
CHAR_O	DC.W	$2020
CHAR_P	DC.W	$2020
CHAR_Q	DC.W	$2020
CHAR_R	DC.W	$2020
CHAR_S	DC.W	$2020
CHAR_T	DC.W	$2020
CHAR_U	DC.W	$2020
CHAR_V	DC.W	$2020
CHAR_W	DC.W	$2020
CHAR_X	DC.W	$2020
CHAR_Y	DC.W	$2020
CHAR_Z	DC.W	$2020
CHAR_	DC.W	$2020
CHAR__	DC.W	$2020
	ORG $3FEE
CHAR2	DC.W	$2020
CHAR3	DC.W	$2020
CHAR4	DC.W	$2020
CHAR5	DC.W	$2020
CHAR6	DC.W	$2020
CHAR7	DC.W	$2020
CHAR8	DC.W	$2020
CHAR9	DC.W	$2020
CHAR10	DC.W	$2020


	END
